import os
from typing import Any, Dict, List, Union
import requests
import bs4
import pandas as pd
import time

"""
Scraping
- Obtaining a list of repositories and packages in the "Dependency graph" in Qiskit/qiskit
"""
def scraping(url: str, result: List[str]) -> List[str]:
    time.sleep(1)
    print(f"> {url}")
    site = requests.get(url)
    soup = bs4.BeautifulSoup(site.text, "html.parser")

    # Get a list of usernames/repository names
    # - For deleted ones, you can get just the name with t.find('span').text
    repository_names: List[str] = [
        "{}/{}".format(
            t.find('a', {"data-repository-hovercards-enabled":""}).text,
            t.find('a', {"data-hovercard-type":"repository"}).text
        ) if (t.find('a', {"data-repository-hovercards-enabled":""}) is not None) else t.find('span').text.strip()
        for t in soup.findAll("div", {"class": "Box-row"})
    ]
    result += repository_names
    print(f"> {len(repository_names)} items retrieved, total of {len(result)} items")

    # Get the URL of the next page
    pagination_container_buttons = soup.find("div", {"class": "paginate-container"}).findAll('a')
    if (pagination_container_buttons[-1].text == "Next"):
        next_url = pagination_container_buttons[-1]["href"]
        # Scraping the next page
        return scraping(next_url, result)
    else:
        return result

"""
main
- Call scraping, save to csv
"""
def main() -> None:
    os.makedirs('./scraping_result', exist_ok=True)
    # Dependency graph: Obtain a list of packages
    print("Dependency graph: Obtaining a list of packages")
    packages = scraping("https://github.com/Qiskit/qiskit/network/dependents?dependent_type=PACKAGE&package_id=UGFja2FnZS0zNTE0NTE5ODI%3D", [])
    df = pd.DataFrame(packages, columns=["packages"])
    df.index = df.index + 1
    df.to_csv('./scraping_result/packages.csv')
    # print the number of packages
    print(f"Number of packages: {len(packages)}")
    
if __name__ == "__main__":
    main()
    